"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.init = void 0;
const inquirer_1 = __importDefault(require("inquirer"));
const globby_1 = __importDefault(require("globby"));
const fs_1 = __importDefault(require("fs"));
// this is compatible with node@12+
const fs = fs_1.default.promises;
const path_1 = __importDefault(require("path"));
// @ts-ignore
const files_1 = require("@covector/files");
const covectorPackageFile = require("../package.json");
// for future typescripting reference
// most of the @ts-ignore have to do with Dir/FileHandle vs string
// and not considering yield correctly?
const init = function* init({ cwd = process.cwd(), changeFolder = ".changes", yes, }) {
    var _a, _b, _c;
    //@ts-ignore
    const answers = yield inquirer_1.default
        .prompt([
        {
            type: "input",
            name: "git url",
            message: "What is the url to your github repo?",
            when: !yes,
            filter: (userInput, answers) => {
                if (userInput.endsWith("/")) {
                    return userInput;
                }
                else {
                    return userInput.length > 0 ? `${userInput}/` : userInput;
                }
            },
        },
        {
            type: "confirm",
            name: "github actions",
            message: "should we include github action workflows?",
            default: true,
            when: !yes,
        },
        {
            type: "input",
            name: "branch name",
            message: "What is the name of your default branch?",
            default: "main",
            when: (answers) => !yes && answers["github actions"],
        },
    ])
        .then((answers) => {
        return Object.assign({ "github actions": true }, answers);
    })
        .catch((error) => {
        throw new Error(error);
    });
    try {
        //@ts-ignore
        const testOpen = yield fs.opendir(path_1.default.posix.join(cwd, changeFolder));
        console.log(`The ${changeFolder} folder exists, skipping creation.`);
        //@ts-ignore
        yield testOpen.close();
    }
    catch (e) {
        console.log(`Creating the ${changeFolder} directory.`);
        //@ts-ignore
        yield fs.mkdir(path_1.default.posix.join(cwd, changeFolder));
    }
    //@ts-ignore
    const pkgs = yield packageFiles({ cwd });
    let packages = {};
    let pkgManagers = {};
    //@ts-ignore
    const pkgFiles = yield Promise.all(pkgs.map((pkg) => files_1.readPkgFile({ file: path_1.default.posix.join(cwd, `${pkg}`), nickname: pkg })));
    for (let pkgFile of pkgFiles) {
        //@ts-ignore
        if (!pkgFile.pkg.workspaces) {
            //@ts-ignore
            const manager = yield derivePkgManager({
                path: path_1.default.dirname(`./${pkgFile.name}`),
                //@ts-ignore
                pkgFile,
            });
            pkgManagers[manager] = true;
            const dependencies = buildDependencyGraph({ pkgFile, pkgFiles });
            //@ts-ignore
            packages[((_a = pkgFile === null || pkgFile === void 0 ? void 0 : pkgFile.pkg) === null || _a === void 0 ? void 0 : _a.name) || ((_c = (_b = pkgFile === null || pkgFile === void 0 ? void 0 : pkgFile.pkg) === null || _b === void 0 ? void 0 : _b.package) === null || _c === void 0 ? void 0 : _c.name)] = Object.assign({ path: path_1.default.dirname(`./${pkgFile.name}`), manager }, (dependencies.length > 0 ? { dependencies } : {}));
        }
    }
    const javascript = {
        version: true,
        getPublishedVersion: "npm view ${ pkgFile.pkg.name } version",
        publish: ["npm publish --access public"],
    };
    const rust = {
        version: true,
        getPublishedVersion: 'cargo search ${ pkg.pkg } --limit 1 | sed -nE \'s/^[^"]*"//; s/".*//1p\' -',
        publish: ["cargo publish"],
    };
    const githubAction = {
        preversion: ["npm install", "npm run build"],
        getPublishedVersion: "git tag v${ pkgFile.version } 2> /dev/null || echo ${ pkgFile.version }",
        publish: [
            "git tag v${ pkgFile.versionMajor } -f",
            "git tag v${ pkgFile.versionMajor }.${ pkgFile.versionMinor } -f",
            "git push --tags -f",
        ],
    };
    const config = Object.assign(Object.assign({}, (answers["git url"] ? { gitSiteUrl: answers["git url"] } : {})), { pkgManagers: Object.assign(Object.assign(Object.assign({}, (pkgManagers.javascript ? { javascript } : {})), (pkgManagers.rust ? { rust } : {})), (pkgManagers["github action"]
            ? { ["github action"]: githubAction }
            : {})), packages });
    // .changes/config.json
    try {
        //@ts-ignore
        const testOpen = yield fs.open(path_1.default.posix.join(cwd, changeFolder, "config.json"), "r");
        console.log(`The config.json exists in ${changeFolder}, skipping creation.`);
        //@ts-ignore
        yield testOpen.close();
    }
    catch (e) {
        console.log("Writing out the config file.");
        //@ts-ignore
        yield fs.writeFile(path_1.default.posix.join(cwd, changeFolder, "config.json"), JSON.stringify(config, null, 2));
    }
    // .changes/readme.md
    try {
        //@ts-ignore
        const testOpen = yield fs.open(path_1.default.posix.join(cwd, changeFolder, "readme.md"), "r");
        console.log(`The readme.md exists in ${changeFolder}, skipping creation.`);
        //@ts-ignore
        yield testOpen.close();
    }
    catch (e) {
        console.log("Writing out a readme to serve as your guide.");
        //@ts-ignore
        yield fs.writeFile(path_1.default.posix.join(cwd, changeFolder, "readme.md"), readme);
    }
    if (answers["github actions"]) {
        const covectorVersionSplit = covectorPackageFile.version.split(".");
        let covectorVersion = `${covectorVersionSplit[0]}.${covectorVersionSplit[1]}`;
        try {
            //@ts-ignore
            const testOpen = yield fs.opendir(path_1.default.posix.join(cwd, "./.github/workflows/"));
            console.log(`The .github/workflows folder exists, skipping creation.`);
            //@ts-ignore
            yield testOpen.close();
        }
        catch (e) {
            console.log(`Creating the .github/workflows directory.`);
            //@ts-ignore
            yield fs.mkdir(path_1.default.posix.join(cwd, "./.github/workflows/"), {
                recursive: true,
            });
        }
        // github status
        try {
            //@ts-ignore
            const testOpen = yield fs.open(path_1.default.posix.join(cwd, ".github", "workflows", "covector-status.yml"), "r");
            console.log(`The status workflow exists in ./.github/workflows, skipping creation.`);
            //@ts-ignore
            yield testOpen.close();
        }
        catch (e) {
            console.log("Writing out covector-status.yml to give you a covector update on PR.");
            //@ts-ignore
            yield fs.writeFile(path_1.default.posix.join(cwd, ".github", "workflows", "covector-status.yml"), githubStatusWorkflow({ version: covectorVersion }));
        }
        // github version and publish
        try {
            //@ts-ignore
            const testOpen = yield fs.open(path_1.default.posix.join(cwd, ".github", "workflows", "covector-version-or-publish.yml"), "r");
            console.log(`The version/publish workflow exists in ./.github/workflows, skipping creation.`);
            //@ts-ignore
            yield testOpen.close();
        }
        catch (e) {
            console.log("Writing out covector-version-or-publish.yml to version and publish your packages.");
            //@ts-ignore
            yield fs.writeFile(path_1.default.posix.join(cwd, ".github", "workflows", "covector-version-or-publish.yml"), githubPublishWorkflow({
                pkgManagers,
                branchName: answers["branch name"],
                version: covectorVersion,
            }));
        }
    }
    return "complete";
};
exports.init = init;
const packageFiles = ({ cwd = process.cwd() }) => __awaiter(void 0, void 0, void 0, function* () {
    return yield globby_1.default(["**/package.json", "**/Cargo.toml", "!**/__fixtures__", "!**/__tests__"], {
        cwd,
        gitignore: true,
    });
});
const derivePkgManager = ({ path, pkgFile, }) => __awaiter(void 0, void 0, void 0, function* () {
    const actionFile = yield globby_1.default(["action.yml"], {
        cwd: path,
        gitignore: true,
    });
    if (actionFile.length > 0) {
        return "github action";
    }
    else {
        return pkgFile.name.endsWith("Cargo.toml") ? "rust" : "javascript";
    }
});
// build dep graph
const buildDependencyGraph = ({ pkgFile, pkgFiles, }) => {
    var _a, _b;
    const pkgDeps = [
        ...(((_a = pkgFile.pkg) === null || _a === void 0 ? void 0 : _a.dependencies) ? Object.keys(pkgFile.pkg.dependencies) : []),
        ...(((_b = pkgFile.pkg) === null || _b === void 0 ? void 0 : _b.devDependencies)
            ? Object.keys(pkgFile.pkg.devDependencies)
            : []),
    ];
    return pkgDeps.reduce((deps, dep) => {
        var _a;
        for (let pkg of pkgFiles) {
            if (dep === ((_a = pkg === null || pkg === void 0 ? void 0 : pkg.pkg) === null || _a === void 0 ? void 0 : _a.name)) {
                return deps.concat([dep]);
            }
        }
        return deps;
    }, []);
};
const readme = `# Changes

##### via https://github.com/jbolda/covector

As you create PRs and make changes that require a version bump, please add a new markdown file in this folder. You do not note the version _number_, but rather the type of bump that you expect: major, minor, or patch. The filename is not important, as long as it is a \`.md\`, but we recommend that it represents the overall change for organizational purposes.

When you select the version bump required, you do _not_ need to consider dependencies. Only note the package with the actual change, and any packages that depend on that package will be bumped automatically in the process.

Use the following format:

\`\`\`md
---
"package-a": patch
"package-b": minor
---

Change summary goes here

\`\`\`

Summaries do not have a specific character limit, but are text only. These summaries are used within the (future implementation of) changelogs. They will give context to the change and also point back to the original PR if more details and context are needed.

Changes will be designated as a \`major\`, \`minor\` or \`patch\` as further described in [semver](https://semver.org/).

Given a version number MAJOR.MINOR.PATCH, increment the:

- MAJOR version when you make incompatible API changes,
- MINOR version when you add functionality in a backwards compatible manner, and
- PATCH version when you make backwards compatible bug fixes.

Additional labels for pre-release and build metadata are available as extensions to the MAJOR.MINOR.PATCH format, but will be discussed prior to usage (as extra steps will be necessary in consideration of merging and publishing).
`;
const githubStatusWorkflow = ({ version = "0", }) => `name: covector status
on: [pull_request]

jobs:
  covector:
    runs-on: ubuntu-latest

    steps:
      - uses: actions/checkout@v2
        with:
          fetch-depth: 0 # required for use of git history
      - name: covector status
        uses: jbolda/covector/packages/action@covector-v${version}
        id: covector
        with:
          command: 'status'
`;
const githubPublishWorkflow = ({ branchName = "main", pkgManagers, version = "0", }) => `name: version or publish

on:
  push:
    branches:
      - ${branchName}

jobs:
  version-or-publish:
    runs-on: ubuntu-latest
    timeout-minutes: 65
    outputs:
      change: \${{ steps.covector.outputs.change }}
      commandRan: \${{ steps.covector.outputs.commandRan }}
      successfulPublish: \${{ steps.covector.outputs.successfulPublish }}

    steps:
      - uses: actions/checkout@v2
        with:
          fetch-depth: 0 # required for use of git history${pkgManagers.javascript
    ? `
      - uses: actions/setup-node@v2
        with:
          node-version: 14
          registry-url: 'https://registry.npmjs.org'`
    : ""}${pkgManagers.rust
    ? `
      - name: cargo login
        run: cargo login \${{ secrets.crate_token }}`
    : ""}
      - name: git config
        run: |
          git config --global user.name "\${{ github.event.pusher.name }}"
          git config --global user.email "\${{ github.event.pusher.email }}"
      - name: covector version or publish (publish when no change files present)
        uses: jbolda/covector/packages/action@covector-v${version}
        id: covector
        env:${pkgManagers.javascript
    ? `
          NODE_AUTH_TOKEN: \${{ secrets.NPM_TOKEN }}`
    : ""}${pkgManagers.rust
    ? `
          CARGO_AUDIT_OPTIONS: \${{ secrets.CARGO_AUDIT_OPTIONS }}`
    : ""}
        with:
          token: \${{ secrets.GITHUB_TOKEN }}
          command: 'version-or-publish'
          createRelease: true
      - name: Create Pull Request With Versions Bumped
        id: cpr
        uses: peter-evans/create-pull-request@v3
        if: steps.covector.outputs.commandRan == 'version'
        with:
          title: "Publish New Versions"
          commit-message: "publish new versions"
          labels: "version updates"
          branch: "release"
          body: \${{ steps.covector.outputs.change }}
`;
