"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.covector = void 0;
const command_1 = require("@covector/command");
const files_1 = require("@covector/files");
const assemble_1 = require("@covector/assemble");
const changelog_1 = require("@covector/changelog");
const apply_1 = require("@covector/apply");
function* covector({ command, dryRun = false, cwd = process.cwd(), filterPackages = [], modifyConfig = (c) => __awaiter(this, void 0, void 0, function* () { return c; }), previewVersion = "", branchTag = "", }) {
    var _a;
    const config = yield modifyConfig(yield files_1.configFile({ cwd }));
    const pre = yield files_1.readPreFile({ cwd, changeFolder: config.changeFolder });
    const prereleaseIdentifier = !pre ? null : pre.tag;
    const changesPaths = yield files_1.changeFiles({
        cwd,
        changeFolder: config.changeFolder,
    });
    const changesVfiles = files_1.changeFilesToVfile({
        cwd,
        paths: changesPaths,
    });
    const assembledChanges = yield assemble_1.assemble({
        cwd,
        vfiles: changesVfiles,
        config,
        preMode: { on: !!pre, prevFiles: !pre ? [] : pre.changes },
    });
    if (command === "status" || !command) {
        if (changesVfiles.length === 0) {
            console.info("There are no changes.");
            const { commands: publishCommands, } = yield assemble_1.mergeIntoConfig({
                assembledChanges,
                config,
                command: "publish",
                cwd,
                dryRun,
                filterPackages,
                tag: branchTag,
            });
            if (publishCommands.length === 0) {
                console.log(`No commands configured to run on publish.`);
                return {
                    response: `No commands configured to run on publish.`,
                    pkgReadyToPublish: [],
                };
            }
            const commandsToRun = yield command_1.confirmCommandsToRun({
                cwd,
                commands: publishCommands,
                command: "publish",
            });
            if (commandsToRun.length > 0) {
                console.log(`There ${commandsToRun.length === 1
                    ? `is 1 package`
                    : `is ${commandsToRun.length} packages`} ready to publish which includes${commandsToRun.map((pkg) => { var _a; return ` ${pkg.pkg}@${(_a = pkg.pkgFile) === null || _a === void 0 ? void 0 : _a.version}`; })}`);
            }
            return {
                pkgReadyToPublish: commandsToRun,
                response: "No changes.",
            };
        }
        else if (!!pre && ((_a = assembledChanges === null || assembledChanges === void 0 ? void 0 : assembledChanges.changes) === null || _a === void 0 ? void 0 : _a.length) === 0) {
            console.info("There are no changes.");
            console.log("We have previously released the changes in these files:", changesPaths);
            return { pkgReadyToPublish: [], response: "No changes." };
        }
        else {
            // write out all of the changes
            // TODO make it pretty
            console.log("changes:");
            Object.keys(assembledChanges.releases).forEach((release) => {
                console.log(`${release} => ${assembledChanges.releases[release].type}`);
                console.dir(assembledChanges.releases[release].changes);
            });
            const changes = apply_1.changesConsideringParents({
                assembledChanges,
                config,
                prereleaseIdentifier,
            });
            const { commands } = yield assemble_1.mergeChangesToConfig({
                assembledChanges: changes,
                config,
                command,
                dryRun,
                filterPackages,
                cwd,
            });
            const applied = yield apply_1.validateApply({
                commands,
                config,
                cwd,
                prereleaseIdentifier,
            });
            return {
                response: `There are ${Object.keys(assembledChanges.releases).length} changes which include${Object.keys(assembledChanges.releases).map((release) => ` ${release} with ${assembledChanges.releases[release].type}`)}`,
            };
        }
    }
    else if (command === "config") {
        delete config.vfile;
        console.dir(config);
        return { response: "config returned" };
    }
    else if (command === "version") {
        yield command_1.raceTime({ t: config.timeout });
        const changes = apply_1.changesConsideringParents({
            assembledChanges,
            config,
            prereleaseIdentifier,
        });
        const { commands, pipeTemplate, } = yield assemble_1.mergeChangesToConfig({
            assembledChanges: changes,
            config,
            command,
            dryRun,
            filterPackages,
            cwd,
        });
        if (dryRun) {
            console.log("==== commands ready to run ===");
            console.log(commands);
        }
        let pkgCommandsRan = Object.keys(config.packages).reduce((pkgs, pkg) => {
            pkgs[pkg] = {
                precommand: false,
                command: false,
                postcommand: false,
                applied: false,
            };
            return pkgs;
        }, {});
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands,
            commandPrefix: "pre",
            command,
            pkgCommandsRan,
            dryRun,
        });
        const applied = yield apply_1.apply({
            //@ts-ignore
            commands,
            config,
            cwd,
            bump: !dryRun,
            prereleaseIdentifier,
        });
        pkgCommandsRan = applied.reduce((pkgs, result) => {
            pkgs[result.name].applied = result;
            return pkgs;
        }, pkgCommandsRan);
        pkgCommandsRan = yield changelog_1.fillChangelogs({
            applied,
            //@ts-ignore
            assembledChanges: changes,
            config,
            cwd,
            pkgCommandsRan,
            create: !dryRun,
        });
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands,
            commandPrefix: "post",
            command,
            pkgCommandsRan,
            dryRun,
        });
        if (command === "version" && !dryRun) {
            if (pre) {
                pre.changes = changesPaths;
                yield files_1.writePreFile({ preFile: pre });
            }
            else {
                yield files_1.changeFilesRemove({ cwd, paths: changesPaths });
            }
        }
        if (dryRun) {
            console.log("==== result ===");
            console.log(pkgCommandsRan);
        }
        return { commandsRan: pkgCommandsRan, pipeTemplate };
    }
    else if (command === "preview") {
        yield command_1.raceTime({ t: config.timeout });
        const versionChanges = apply_1.changesConsideringParents({
            assembledChanges,
            config,
            prereleaseIdentifier,
        });
        const { commands: versionCommands } = yield assemble_1.mergeChangesToConfig({
            assembledChanges: versionChanges,
            config,
            command: "version",
            dryRun,
            filterPackages,
            cwd,
        });
        let pkgCommandsRan = Object.keys(config.packages).reduce((pkgs, pkg) => {
            pkgs[pkg] = {
                precommand: false,
                command: false,
                postcommand: false,
                applied: false,
            };
            return pkgs;
        }, {});
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands: versionCommands,
            commandPrefix: "pre",
            command: "version",
            pkgCommandsRan,
            dryRun,
        });
        const applied = yield apply_1.apply({
            commands: versionCommands,
            config,
            cwd,
            bump: !dryRun,
            previewVersion,
            prereleaseIdentifier,
        });
        pkgCommandsRan = applied.reduce((pkgs, result) => {
            pkgs[result.name].applied = result;
            return pkgs;
        }, pkgCommandsRan);
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands: versionCommands,
            commandPrefix: "post",
            command: "version",
            pkgCommandsRan,
            dryRun,
        });
        const { commands: publishCommands, } = yield assemble_1.mergeIntoConfig({
            assembledChanges,
            config,
            command: "publish",
            cwd,
            dryRun,
            filterPackages,
            tag: branchTag,
        });
        if (publishCommands.length === 0) {
            console.log(`No commands configured to run on publish.`);
            return {
                response: `No commands configured to run on publish.`,
            };
        }
        const commandsToRun = yield command_1.confirmCommandsToRun({
            cwd,
            commands: publishCommands,
            command: "publish",
        });
        pkgCommandsRan = publishCommands.reduce((pkgs, pkg) => {
            pkgs[pkg.pkg] = {
                precommand: false,
                command: false,
                postcommand: false,
                pkg,
            };
            return pkgs;
        }, {});
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands: commandsToRun,
            commandPrefix: "pre",
            command: "publish",
            pkgCommandsRan,
            dryRun,
        });
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands: commandsToRun,
            command: "publish",
            pkgCommandsRan,
            dryRun,
        });
        return { commandsRan: pkgCommandsRan, pipeTemplate: publishCommands };
    }
    else {
        yield command_1.raceTime({ t: config.timeout });
        const changelogs = yield changelog_1.pullLastChangelog({
            config,
            cwd,
        });
        const { commands, pipeTemplate, } = yield assemble_1.mergeIntoConfig({
            assembledChanges,
            config,
            command,
            cwd,
            dryRun,
            filterPackages,
            changelogs,
        });
        if (dryRun) {
            console.log("==== commands ready to run ===");
            console.log(commands);
        }
        if (commands.length === 0) {
            console.log(`No commands configured to run on [${command}].`);
            return {
                response: `No commands configured to run on [${command}].`,
            };
        }
        const commandsToRun = yield command_1.confirmCommandsToRun({
            cwd,
            commands,
            command,
        });
        let pkgCommandsRan = commands.reduce((pkgs, pkg) => {
            pkgs[pkg.pkg] = {
                precommand: false,
                command: false,
                postcommand: false,
                pkg,
            };
            return pkgs;
        }, {});
        pkgCommandsRan = yield changelog_1.pipeChangelogToCommands({
            changelogs,
            pkgCommandsRan,
        });
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands: commandsToRun,
            commandPrefix: "pre",
            command,
            pkgCommandsRan,
            dryRun,
        });
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands: commandsToRun,
            command,
            pkgCommandsRan,
            dryRun,
        });
        pkgCommandsRan = yield command_1.attemptCommands({
            cwd,
            commands: commandsToRun,
            commandPrefix: "post",
            command,
            pkgCommandsRan,
            dryRun,
        });
        if (dryRun) {
            console.log("==== result ===");
            console.log(pkgCommandsRan);
        }
        return { commandsRan: pkgCommandsRan, pipeTemplate };
    }
}
exports.covector = covector;
